/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */






#include "smartsearch.h"

#include <sqlite3.h>

#include <contacts-svc.h>
#include <calendar-svc-provider.h>
#include <memo-db.h>

#include <email-api-mail.h>

#include <favorites.h>
#include <media_content.h>

#include <msg_storage.h>
#include <msg_storage_types.h>
#include <msg.h>

#include <ail.h>
#include <unicode/ucal.h>
#include <unicode/ustring.h>
#include <unicode/ustdio.h>
#include <unicode/udat.h>

char* g_keyword = NULL;

struct search_cb_data {
	struct search_content_object *obj;
	struct appdata *ad;
	char *keyword;
	struct search_genlist_info *result_info;
};

struct search_cb_data_browser {
	struct search_content_object *obj;
	struct appdata *ad;
	char *keyword;
	struct search_genlist_info *result_info;
	struct search_more_result *info;
};

struct media_cb_data {
	bool is_sdcard;
	char *dirname;
};

int search_sql_stmt_init(sqlite3_stmt * sql_stmt)
{
	SEARCH_FUNC_START;

	int rc = 0;

	rc = sqlite3_clear_bindings(sql_stmt);
	if (rc != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("sqlite3_clear_bindings error");
		return SEARCH_RET_FAIL;
	}

	rc = sqlite3_reset(sql_stmt);
	if (rc != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("sqlite3_reset error");
		return SEARCH_RET_FAIL;
	}

	SEARCH_FUNC_END;

	return SEARCH_RET_SUCCESS;
}

int search_sql_stmt_finalize(sqlite3_stmt * stmt)
{
	SEARCH_FUNC_START;

	int ret;

	ret = sqlite3_finalize(stmt);
	if (ret != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("sqlite3_finalize error : %d ", ret);
		return -1;
	}

	SEARCH_FUNC_END;

	return 0;
}

int search_sql_prepare_stmt(sqlite3 *db_handle, const char *query,
					 sqlite3_stmt ** stmt_out)
{
	SEARCH_FUNC_START;

	int rc = 0;
	int ret = 0;

	rc = sqlite3_prepare(db_handle, query, strlen(query), stmt_out, NULL);
	if (rc != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("sql preapre error : %d / %s", rc, query);
		ret = SEARCH_RET_FAIL;
	} else {
		ret = SEARCH_RET_SUCCESS;
	}

	SEARCH_FUNC_END;

	return ret;
}

int search_sql_bind_text(sqlite3* db_handle, sqlite3_stmt *sql_stmt, int col, char *data)
{
	int rc = 0;
	int ret = SEARCH_RET_SEARCH_NONE;

	rc = sqlite3_bind_text(sql_stmt, col, data, strlen(data), SQLITE_STATIC);
	if (rc != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("SQL BIND TEXT ERROR : %s(%d) / %s\n",
			     sqlite3_errmsg(db_handle),
			     rc,
			     data);
		ret = SEARCH_RET_SEARCH_FAIL;
	}

	return ret;
}

int search_sql_bind_int(sqlite3* db_handle, sqlite3_stmt *sql_stmt, int col, int data)
{
	int rc = 0;
	int ret = SEARCH_RET_SEARCH_NONE;

	rc = sqlite3_bind_int(sql_stmt, col, data);
	if (rc != SQLITE_OK) {
		SEARCH_DEBUG_ERROR("SQL BIND INT ERROR : %s(%d) / %d\n",
			     sqlite3_errmsg(db_handle),
			     rc,
			     data);
		ret = SEARCH_RET_SEARCH_FAIL;
	}
	return ret;
}

static int __search_sql_contact_search_result_cb(CTSvalue * value,
						 void *user_data)
{
	SEARCH_FUNC_START;

	struct search_cb_data *scd = (struct search_cb_data*)user_data;

	struct appdata *ad = scd->ad;
	struct search_content_object* obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;
	struct search_genlist_item_info *gen_item = NULL;

	const char* contact_img_path = NULL;
	char display_name[128] = { 0, };
	const char *first, *last, *display, *number;

	bool is_word_in_str;

	if (obj->mr_info.next_point == 0) {
		//### Set group title
		result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
													obj,
													ad->itc_grouptitle,
					    							obj,
													NULL);
	}

	if (obj->mr_info.next_point <
	    obj->mr_info.offset + SEARCH_CATEGORY_LIST_MORE_CNT) {
		sel_mem_info =
		    (struct search_item_sel *)calloc(1,
						     sizeof(struct
							    search_item_sel));
		sel_mem_info->type = obj->contents_type;

		contact_img_path = contacts_svc_value_get_str(value,
						       				CTS_LIST_NUM_CONTACT_IMG_PATH_STR);

		if (contact_img_path != NULL) {
			strncpy(sel_mem_info->icon1_path, contact_img_path,
					MAX_LENGTH_PER_PATH - 1);
	}
		else if (obj->obj_icon_path)
			strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
					MAX_LENGTH_PER_PATH - 1);

		/************ START : set search result to sel_mem_info buf ************/

		/***********************************************************************/

		memset(display_name, 0x00, sizeof(display_name));

		display =
		    contacts_svc_value_get_str(value,
					       CTS_LIST_NUM_CONTACT_DISPLAY_STR);

		if (display) {
			snprintf(display_name, sizeof(display_name), "%s",
				 display);
		} else {
			first =
			    contacts_svc_value_get_str(value,
						       CTS_LIST_NUM_CONTACT_FIRST_STR);
			last =
			    contacts_svc_value_get_str(value,
						       CTS_LIST_NUM_CONTACT_LAST_STR);

			if (CTS_ORDER_NAME_FIRSTLAST ==
			    contacts_svc_get_order(CTS_ORDER_OF_DISPLAY)) {
				if (first) {
					if (last) {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s %s", first, last);
					} else {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s", first);
					}
				} else {
					if (last) {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s", last);
					}
				}
			} else {
				if (last) {
					if (first) {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s %s", last, first);
					} else {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s", last);
					}
				} else {
					if (first) {
						snprintf(display_name,
							 sizeof(display_name),
							 "%s", first);
					}
				}
			}
		}

		SEARCH_DEBUG_LOG("display_name : %s", display_name);

		if(strlen(display_name) > 0) {
			strncpy(sel_mem_info->main_buf,
					search_markup_keyword(display_name, keyword, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		/***********************************************************************/

		number = contacts_svc_value_get_str(value, CTS_LIST_NUM_NUMBER_STR);

		if((number) && strlen(number) > 0) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(number, keyword, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		/***********************************************************************/

		snprintf(sel_mem_info->main_id, MAX_LENGTH_PER_ID - 1, "%d",
			 contacts_svc_value_get_int(value,
						    CTS_LIST_NUM_CONTACT_ID_INT));
		SEARCH_DEBUG_LOG("Search CT_ID: %d", sel_mem_info->main_id);
		/***********************************************************************/
		/************ END : set search result to sel_mem_info buf **************/
		gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
										obj,
										ad->itc_pslist_2line_1icon,
					sel_mem_info,
										obj->gl_result_grp);
		result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
		obj->mr_info.next_point++;
	} else if (obj->mr_info.next_point ==
		   obj->mr_info.offset +
		   SEARCH_CATEGORY_LIST_MORE_CNT) {
		/* Set more title to genlist (under 'search all') */
		sel_mem_more =
		    (struct search_item_more_sel *)calloc(1,
						     sizeof(struct
							    search_item_more_sel));
		sel_mem_more->obj = obj;

		result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
												obj,
												ad->itc_listmore,
												sel_mem_more,
												obj->gl_result_grp);
	} else {
		SEARCH_DEBUG_WARNING
		    ("__search_sql_contact_search_result_cb wrong index : %d",
		     obj->mr_info.next_point);
	}

	SEARCH_FUNC_END;

	return 0;
}

int search_sql_contact_search_result_by_api(struct search_content_object *obj,
		int offset, int limit, char *keyword, void *data, struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	struct search_cb_data scd = {obj, ad, keyword, result_info};

	int ret = 0;

	/* Check whether search result is exist or not */
	ret =
	    contacts_svc_smartsearch_excl(keyword, limit, offset,
					  __search_sql_contact_search_result_cb,
					  &scd);
	if (ret != CTS_SUCCESS) {
		SEARCH_DEBUG_WARNING("contacts svc search return error : %d",
				     ret);
		return SEARCH_RET_SEARCH_FAIL;
	}

	if (obj->mr_info.next_point == 0) {
		SEARCH_DEBUG_WARNING("contacts svc search no result");
		return SEARCH_RET_SEARCH_NONE;
	}

	SEARCH_FUNC_END;

	return SEARCH_RET_SEARCH_SUCCESS;
}

int search_sql_calendar_search_result_by_api(struct search_content_object *obj,
		int offset, int limit, char *keyword, void *data, struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	int ret;
	struct appdata *ad = (struct appdata *)data;

	cal_struct *event = NULL;
	cal_iter *iter = NULL;

	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;
	struct search_genlist_item_info *gen_item = NULL;

	int loop_cnt = 0;

	char *summary = NULL;
	bool is_word_in_str;

	int time_type = 0;

	long long int start_time = 0;
	UErrorCode status = U_ZERO_ERROR;
	UCalendar *cal;
	int y = 0, m = 0, d = 0;

	ret = calendar_svc_smartsearch_excl(keyword, offset, limit, &iter);

	if (ret == CAL_SUCCESS) {
		ret = calendar_svc_iter_next(iter);
		if (ret != CAL_SUCCESS) {
			SEARCH_DEBUG_WARNING
			    ("calendar svc search no result : %d", ret);
			return SEARCH_RET_SEARCH_NONE;
		}
		/* Set group title */
		if (offset == 0) {
			result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
														obj,
														ad->itc_grouptitle,
						    							obj,
														NULL);
		}
		/* Set search result to genlist */
		do {
			loop_cnt++;

			if (loop_cnt == limit) {
				break;
			}
			/********** START : set search result to sel_mem_info buf ************/

			ret = calendar_svc_iter_get_info(iter, &event);

			if (ret == CAL_SUCCESS) {
				sel_mem_info =
				    (struct search_item_sel *)calloc(1,
								     sizeof
								     (struct
								      search_item_sel));
				sel_mem_info->type = obj->contents_type;

			if (obj->obj_icon_path)
				strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
						MAX_LENGTH_PER_PATH - 1);

				summary =
				    calendar_svc_struct_get_str(event,
								CAL_VALUE_TXT_SUMMARY);

				if ((summary) && (strlen(summary) > 0)) {
					strncpy(sel_mem_info->main_buf,
						search_markup_keyword(summary,
								     keyword,
									     &is_word_in_str),
							MAX_LENGTH_PER_LINE - 1);
				}

				snprintf(sel_mem_info->main_id,
					 MAX_LENGTH_PER_ID - 1, "%d",
					 calendar_svc_struct_get_int(event,
								     CAL_VALUE_INT_INDEX));

				time_type = calendar_svc_struct_get_int(event, CALS_VALUE_INT_DTSTART_TYPE);

				if (time_type == CALS_TIME_LOCALTIME) {
					y = calendar_svc_struct_get_int(event, CALS_VALUE_INT_DTSTART_YEAR);
					m = calendar_svc_struct_get_int(event, CALS_VALUE_INT_DTSTART_MONTH);
					d = calendar_svc_struct_get_int(event, CALS_VALUE_INT_DTSTART_MDAY);
				} else if (time_type == CALS_TIME_UTIME) {
					start_time = calendar_svc_struct_get_lli(event,
									CALS_VALUE_LLI_DTEND_UTIME);
					UChar *_tzid;
					_tzid = (UChar *)malloc(sizeof(UChar) * (strlen("Europe/Berlin") + 1));
					if (_tzid) u_uastrcpy(_tzid, "Europe/Berlin");

					cal = ucal_open(_tzid, u_strlen(_tzid), "en_US", UCAL_TRADITIONAL, &status);
					ucal_setMillis(cal, (double)(start_time * 1000.0), &status);
					y = ucal_get(cal, UCAL_YEAR, &status);
					m = ucal_get(cal, UCAL_MONTH, &status);
					d = ucal_get(cal, UCAL_DATE, &status);
					ucal_close(cal);
				}
				snprintf(sel_mem_info->sub_buf, MAX_LENGTH_PER_LINE, "%d.%d.%d", y,m,d);

				calendar_svc_struct_free(&event);
				summary = NULL;

				gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
												obj,
												ad->itc_pslist_2line_1icon,
							sel_mem_info,
												obj->gl_result_grp);
				result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
			} else {
				SEARCH_DEBUG_WARNING
				    ("calendar_svc_iter_get_info fail : %d",
				     ret);
				break;
			}

		}
		while (calendar_svc_iter_next(iter) == CAL_SUCCESS);
	} else {
		SEARCH_DEBUG_WARNING("calendar_svc_search_list fail : %d", ret);
		return SEARCH_RET_SEARCH_FAIL;
	}

	/* Set more title to genlist (under 'search all') */
	if (loop_cnt == limit) {
		obj->mr_info.next_point = offset + limit - 1;

		sel_mem_more =
		    (struct search_item_more_sel *)calloc(1,
						     sizeof(struct
							    search_item_more_sel));
		sel_mem_more->obj = obj;

		result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
												obj,
												ad->itc_listmore,
												sel_mem_more,
												obj->gl_result_grp);
	}

	SEARCH_FUNC_END;
	return SEARCH_RET_SEARCH_SUCCESS;
}

static void __search_sql_memo_search_result_cb(memo_data_t * value,
					       void *user_data)
{
	SEARCH_FUNC_START;

	struct search_cb_data *scd = (struct search_cb_data *)user_data;
	struct appdata *ad = scd->ad;
	struct search_content_object *obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;

	struct search_genlist_item_info *gen_item = NULL;
	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;

	char *keyword = scd->keyword;

	bool is_word_in_str;

	if(!value) {
		SEARCH_DEBUG_WARNING("memo data value is null");
		return;
	}

	if (obj->mr_info.next_point == 0) {
		/* Set group title */
		result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
													obj,
													ad->itc_grouptitle,
					    							obj,
													NULL);
	}

	if (obj->mr_info.next_point <
	    obj->mr_info.offset + SEARCH_CATEGORY_LIST_MORE_CNT) {
		sel_mem_info =
		    (struct search_item_sel *)calloc(1,
						     sizeof(struct
							    search_item_sel));
		sel_mem_info->type = obj->contents_type;

		/********** START : set search result to sel_mem_info buf ************/

		if (obj->obj_icon_path)
			strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
					MAX_LENGTH_PER_PATH - 1);

		/* First, We should check comment field. It means drawing memo
		 * If comment field is NULL, then we check content field */
		if ((value->comment) && (strlen(value->comment) > 0)) {
			strncpy(sel_mem_info->main_buf,
						search_markup_keyword(value->comment, keyword,
											&is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		} else if ((value->content) && (strlen(value->content) > 0)) {
			strncpy(sel_mem_info->main_buf,
				search_markup_keyword(value->content, keyword,
				     						&is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}
		snprintf(sel_mem_info->main_id, MAX_LENGTH_PER_ID - 1, "%d", value->id);

		search_util_date_time_format_get_val(localtime
						     (&value->modi_time),
						     sel_mem_info->sub_buf);

		/********** END : set search result to sel_mem_info buf *************/

		gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
										obj,
										ad->itc_pslist_2line_1icon,
					sel_mem_info,
										obj->gl_result_grp);
		result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);

		obj->mr_info.next_point++;
	} else if (obj->mr_info.next_point ==
		   obj->mr_info.offset + SEARCH_CATEGORY_LIST_MORE_CNT) {
		/* Set more title to genlist (under 'search all') */
		sel_mem_more =
		    (struct search_item_more_sel *)calloc(1,
						     sizeof(struct
							    search_item_more_sel));
		sel_mem_more->obj = obj;

		result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
												obj,
												ad->itc_listmore,
												sel_mem_more,
												obj->gl_result_grp);
	} else {
		SEARCH_DEBUG_WARNING
		    ("__search_sql_memo_search_result_cb wrong index : %d",
		     obj->mr_info.next_point);
	}

	SEARCH_FUNC_END;

	return;
}

int search_sql_memo_search_result_by_api(struct search_content_object *obj,
		int offset, int limit, char *keyword, void *data, struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	struct search_cb_data scd = {obj, ad, keyword, result_info};

	memo_search_data(keyword,
					limit, offset,
					MEMO_SORT_TITLE_ASC,
					__search_sql_memo_search_result_cb,
					&scd);

	if (obj->mr_info.next_point == 0) {
		SEARCH_DEBUG_WARNING("no memo search result");
		return SEARCH_RET_SEARCH_NONE;
	}

	SEARCH_FUNC_END;
	return SEARCH_RET_SEARCH_SUCCESS;
}

int search_sql_email_search_result_by_api(struct search_content_object *obj,
		int offset, int limit, char *keyword, void *data, struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;

	int ret = EMAIL_ERROR_NONE;
	int i = 0;
	email_mail_list_item_t *mail_list = NULL;
	email_list_filter_t *filter_list = NULL;
	email_list_sorting_rule_t *sorting_rule_list = NULL;

	int filter_list_count = 5;
	int sorting_rule_list_count = 1;
	int mail_count = 0;

	struct search_genlist_item_info *gen_item = NULL;
	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;
	bool is_word_in_str;

	filter_list = (email_list_filter_t *)malloc(sizeof(email_list_filter_t) * filter_list_count);
	memset(filter_list, 0 , sizeof(email_list_filter_t) * filter_list_count);

	filter_list[0].list_filter_item_type                               = EMAIL_LIST_FILTER_ITEM_RULE;
	filter_list[0].list_filter_item.rule.target_attribute              = EMAIL_MAIL_ATTRIBUTE_SUBJECT;
	filter_list[0].list_filter_item.rule.rule_type                     = EMAIL_LIST_FILTER_RULE_INCLUDE;
	filter_list[0].list_filter_item.rule.key_value.string_type_value   = keyword;
	filter_list[0].list_filter_item.rule.case_sensitivity              = EMAIL_CASE_INSENSITIVE;

	filter_list[1].list_filter_item_type                               = EMAIL_LIST_FILTER_ITEM_OPERATOR;
	filter_list[1].list_filter_item.operator_type                      = EMAIL_LIST_FILTER_OPERATOR_OR;

	filter_list[2].list_filter_item_type                               = EMAIL_LIST_FILTER_ITEM_RULE;
	filter_list[2].list_filter_item.rule.target_attribute              = EMAIL_MAIL_ATTRIBUTE_TO;
	filter_list[2].list_filter_item.rule.rule_type                     = EMAIL_LIST_FILTER_RULE_INCLUDE;
	filter_list[2].list_filter_item.rule.key_value.string_type_value   = keyword;
	filter_list[2].list_filter_item.rule.case_sensitivity              = EMAIL_CASE_INSENSITIVE;

	filter_list[3].list_filter_item_type                               = EMAIL_LIST_FILTER_ITEM_OPERATOR;
	filter_list[3].list_filter_item.operator_type                      = EMAIL_LIST_FILTER_OPERATOR_OR;

	filter_list[4].list_filter_item_type                               = EMAIL_LIST_FILTER_ITEM_RULE;
	filter_list[4].list_filter_item.rule.target_attribute              = EMAIL_MAIL_ATTRIBUTE_FROM;
	filter_list[4].list_filter_item.rule.rule_type                     = EMAIL_LIST_FILTER_RULE_INCLUDE;
	filter_list[4].list_filter_item.rule.key_value.string_type_value   = keyword;
	filter_list[4].list_filter_item.rule.case_sensitivity              = EMAIL_CASE_INSENSITIVE;

	sorting_rule_list = (email_list_sorting_rule_t *)malloc(sizeof(email_list_sorting_rule_t) * sorting_rule_list_count);
	memset(sorting_rule_list, 0, sizeof(email_list_sorting_rule_t) * sorting_rule_list_count);

	sorting_rule_list[0].target_attribute                              = EMAIL_MAIL_ATTRIBUTE_SUBJECT;
	sorting_rule_list[0].sort_order                                    = EMAIL_SORT_ORDER_ASCEND;

	ret = email_get_mail_list_ex(filter_list, filter_list_count, sorting_rule_list, sorting_rule_list_count,
							offset, limit, &mail_list, &mail_count);


	if (ret != EMAIL_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("no email search result");
		return SEARCH_RET_SEARCH_FAIL;
	}

	if (mail_count == 0) {
		SEARCH_DEBUG_WARNING("no email search result");
		return SEARCH_RET_SEARCH_NONE;
	}

	if(mail_list == NULL) {
		SEARCH_DEBUG_WARNING("mail_list is null");
		return SEARCH_RET_SEARCH_NONE;
	}

	/* Set group title */
	if (offset == 0) {
		result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
									obj,
									ad->itc_grouptitle,
									obj,
									NULL);
	}

	/* Set search result to genlist */
	for(i = 0 ; i < mail_count ; i++) {

		if(i == limit -1)
			break;

		sel_mem_info = (struct search_item_sel *)calloc(1,
								     sizeof(struct search_item_sel));

		sel_mem_info->type = obj->contents_type;

		if (obj->obj_icon_path)
			strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
					MAX_LENGTH_PER_PATH - 1);

		snprintf(sel_mem_info->main_id, MAX_LENGTH_PER_ID - 1, "%d", mail_list[i].mail_id);

		snprintf(sel_mem_info->path, MAX_LENGTH_PER_PATH - 1, "%d %d %d",
				mail_list[i].account_id, mail_list[i].mail_id, mail_list[i].mailbox_id);

		SEARCH_DEBUG_LOG("subject : %s", mail_list[i].subject);

		if((mail_list[i].subject) && (strlen(mail_list[i].subject) > 0)) {
			strncpy(sel_mem_info->main_buf,
					search_markup_keyword(mail_list[i].subject, ad->search_word, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		if((mail_list[i].from) && (strlen(mail_list[i].from) > 0)) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(mail_list[i].from, ad->search_word, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		if(is_word_in_str == false && (mail_list[i].recipients) && (strlen(mail_list[i].recipients) > 0)) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(mail_list[i].recipients, ad->search_word, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
							obj,
							ad->itc_pslist_2line_1icon,
						sel_mem_info,
							obj->gl_result_grp);
		result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
	}

	SEARCH_FREE(mail_list);

	/* Set more title to genlist (under 'search all') */
	if (mail_count == limit) {
		obj->mr_info.next_point = offset + mail_count - 1;

		sel_mem_more =
		    (struct search_item_more_sel *)calloc(1,
						     sizeof(struct search_item_more_sel));
		sel_mem_more->obj = obj;

		result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
									obj,
									ad->itc_listmore,
									sel_mem_more,
									obj->gl_result_grp);
	}

	SEARCH_FREE(filter_list);
	SEARCH_FREE(sorting_rule_list);

	SEARCH_FUNC_END;
	return SEARCH_RET_SEARCH_SUCCESS;
}

static bool __search_media_info_cb(media_info_h media, void *user_data)
{
	char *media_id = NULL;
	char *media_name = NULL;
	char *media_path = NULL;
	char *media_thumbnail = NULL;

	struct search_item_sel *sel_mem_info = NULL;
	Elm_Genlist_Item_Class *itc_style = NULL;
	bool is_word_in_str = false;

	struct search_cb_data *scd = (struct search_cb_data*)user_data;
	struct appdata *ad = scd->ad;
	struct search_content_object *obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_genlist_item_info *gen_item = NULL;

	media_info_get_media_id(media, &media_id);
	media_info_get_display_name(media, &media_name);
	media_info_get_file_path(media, &media_path);
	media_info_get_thumbnail_path(media, &media_thumbnail);

	sel_mem_info = (struct search_item_sel *)calloc(1,
						     sizeof(struct search_item_sel));

	sel_mem_info->type = obj->contents_type;

	if ((media_thumbnail) && (strlen(media_thumbnail) > 0)
		&& ecore_file_exists(media_thumbnail) == EINA_TRUE) {
		strncpy(sel_mem_info->icon1_path, media_thumbnail,
				MAX_LENGTH_PER_PATH - 1);
	} else if (obj->obj_icon_path)
		strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
				MAX_LENGTH_PER_PATH - 1);

	if (media_name && strlen(media_name)) {
		strncpy(sel_mem_info->main_buf,
				search_markup_keyword(media_name,
									keyword,
									&is_word_in_str),
				MAX_LENGTH_PER_LINE - 1);
	}

	if (media_id)
		snprintf(sel_mem_info->main_id, MAX_LENGTH_PER_ID, "%s",
			 media_id);

	if (media_path)
		snprintf(sel_mem_info->path, MAX_LENGTH_PER_PATH - 1, "%s",
			 media_path);

	itc_style = ad->itc_pslist_1line_1icon;

	/* Add Info to genlist */
	gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
					obj,
					itc_style,
					sel_mem_info,
					obj->gl_result_grp);
	result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);

	SEARCH_FREE(media_id);
	SEARCH_FREE(media_name);
	SEARCH_FREE(media_thumbnail);
	SEARCH_FREE(media_path);

	return true;
	}

static bool  __search_sql_music_info_cb(media_info_h media, void *user_data)
{
	char *media_id = NULL;
	char *media_path = NULL;
	char *media_thumbnail = NULL;
	char *title = NULL;
	char *artist = NULL;
	char *album = NULL;

	audio_meta_h a_handle;

	media_content_storage_e storage_type = MEDIA_CONTENT_STORAGE_INTERNAL;

	struct search_item_sel *sel_mem_info = NULL;
	Elm_Genlist_Item_Class *itc_style = NULL;
	bool is_word_in_str = false;

	struct search_cb_data *scd = (struct search_cb_data*)user_data;
	struct appdata *ad = scd->ad;
	struct search_content_object* obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_genlist_item_info *gen_item = NULL;

	media_info_get_media_id(media, &media_id);
	media_info_get_file_path(media, &media_path);
	media_info_get_storage_type(media, &storage_type);
	media_info_get_thumbnail_path(media, &media_thumbnail);

	media_info_get_audio(media, &a_handle);
	audio_meta_get_title(a_handle, &title);
	audio_meta_get_album(a_handle, &album);
	audio_meta_get_artist(a_handle, &artist);

	sel_mem_info = (struct search_item_sel *)calloc(1,
					     sizeof(struct search_item_sel));

	sel_mem_info->type = obj->contents_type;;

	if ((media_thumbnail) && (strlen(media_thumbnail) > 0)
		&& ecore_file_exists(media_thumbnail) == EINA_TRUE) {
		strncpy(sel_mem_info->icon1_path , media_thumbnail,
			MAX_LENGTH_PER_PATH -1 );
	} else if(obj->obj_icon_path) {
		strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
			MAX_LENGTH_PER_PATH - 1);
		}

	if ((title) && (strlen(title) > 0)) {
		strncpy(sel_mem_info->main_buf,
				search_markup_keyword(title, keyword, &is_word_in_str),
				MAX_LENGTH_PER_LINE - 1);
	}

	if (is_word_in_str) {
		if ((artist) && (strlen(artist) > 0)) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(artist,
									keyword,
									&is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}
	} else {
		memset(sel_mem_info->sub_buf, '\0', MAX_LENGTH_PER_LINE);

		if ((artist) && (strlen(artist) > 0)) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(artist,
								keyword,
								&is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		if (is_word_in_str == false && (album) && (strlen(album) > 0)) {
			strncpy(sel_mem_info->sub_buf,
					search_markup_keyword(album,
								keyword,
											&is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);
		}

		if (is_word_in_str == false) {
			SEARCH_DEBUG_LOG("search_result is 0. No match keyword in title,album,artist string");
		}
	}

	if (storage_type == MEDIA_CONTENT_STORAGE_EXTERNAL) {
		itc_style = ad->itc_pslist_2line_2icon;
		strncpy(sel_mem_info->icon2_path, SEARCH_SDCARD_ICON,
				MAX_LENGTH_PER_PATH - 1);
	}
	else {
		itc_style = ad->itc_pslist_2line_1icon;
	}

	if (media_id)
		snprintf(sel_mem_info->main_id,
				MAX_LENGTH_PER_ID - 1, "%s", media_id);

	if (media_path)
		strncpy(sel_mem_info->path, media_path, MAX_LENGTH_PER_PATH);

	/* Add Information to genlist */
	gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
									obj,
				itc_style,
						sel_mem_info,
									obj->gl_result_grp);
	result_info->itemlist =  g_list_append(result_info->itemlist, (void*)gen_item);

	SEARCH_FREE(media_id);
	SEARCH_FREE(media_path);
	SEARCH_FREE(media_thumbnail);
	SEARCH_FREE(title);
	SEARCH_FREE(artist);
	SEARCH_FREE(album);

	return true;
	}

static int __search_sql_media_search_result_by_api(struct search_content_object *obj,
																		int offset, int limit,
																		char *keyword,
																		filter_h filter,
																		media_info_cb func_cb,
					    												void *data,
					    												struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	struct search_cb_data scd = {obj, ad, keyword, result_info};

	int result_cnt = 0;
	int ret = 0;
	int func_ret = SEARCH_RET_SEARCH_SUCCESS;

	struct search_item_more_sel *sel_mem_more = NULL;

	media_info_get_media_count_from_db(filter, &result_cnt);

	if (result_cnt != 0) {
		/* Set group title */
		if (obj->mr_info.offset== 0) {
			result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
														obj,
														ad->itc_grouptitle,
														obj,
														NULL);
	}

		ret = media_info_foreach_media_from_db(filter, func_cb, &scd);

		if (ret != MEDIA_CONTENT_ERROR_NONE) {
			SEARCH_DEBUG_WARNING("media_info_foreach_media_from_db failed : %d\n", ret);
			func_ret = SEARCH_RET_SEARCH_FAIL;
			goto out_func;
		}

	/* Set more title to genlist (under 'search all') */
	if (result_cnt == limit) {
			obj->mr_info.next_point = offset + result_cnt - 1;

			sel_mem_more =
			    (struct search_item_more_sel *)calloc(1,
							     sizeof(struct search_item_more_sel));
			sel_mem_more->obj = obj;

			result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
													obj,
													ad->itc_listmore,
													sel_mem_more,
													obj->gl_result_grp);
		}
	} else {
		func_ret = SEARCH_RET_SEARCH_NONE;
	}

out_func:
	SEARCH_FUNC_END;

	return func_ret;
}


int search_sql_image_search_result_by_api(struct search_content_object *obj,
																		int offset, int limit,
																		char *keyword,
					    												void *data,
					    												struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	int ret = SEARCH_RET_SEARCH_SUCCESS;
	int len = 0;
	char *search_query = NULL;
	char *escape_added_keyword = NULL;

	const char *image_query = "(MEDIA_TYPE=0 and MEDIA_DISPLAY_NAME like \"%%%s%%\" ESCAPE (\"%s\"))";

	SEARCH_MALLOC(escape_added_keyword, strlen(keyword)*2+5, char);
	memset(escape_added_keyword, 0x0, strlen(keyword)*2+5);
	search_sql_make_keyword_bind_value(keyword, escape_added_keyword, SEARCH_SQL_BIND_TYPE_DUPLEX);

	len = strlen(escape_added_keyword) + strlen(DB_ESCAPE_CHAR) + strlen(image_query) + 1;
	search_query = (char*)calloc(1, len);

	snprintf(search_query, len, image_query, escape_added_keyword, "%");

	SEARCH_DEBUG_WARNING("keyword :%s", keyword);
	SEARCH_DEBUG_WARNING("search_query :%s", escape_added_keyword);

	filter_h filter;

	ret = media_filter_create(&filter);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_create failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = media_filter_set_offset(filter, offset, limit);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_offset failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = media_filter_set_condition(filter, search_query, MEDIA_CONTENT_COLLATE_DEFAULT);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_condition failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
}

	ret = media_filter_set_order(filter, MEDIA_CONTENT_ORDER_ASC, MEDIA_DISPLAY_NAME, MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_order failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}


	ret = __search_sql_media_search_result_by_api(obj,
												offset,
												limit,
												keyword,
												filter,
												__search_media_info_cb,
												data,
												result_info);

out_func:
	SEARCH_FREE(search_query);
	SEARCH_FREE(escape_added_keyword);
	media_filter_destroy(filter);

	SEARCH_FUNC_END;

	return ret;
}

int search_sql_video_search_result_by_api(struct search_content_object *obj,
		int offset, int limit, char *keyword, void *data, struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	int ret = SEARCH_RET_SEARCH_SUCCESS;
	int len = 0;
	char *search_query = NULL;
	char *escape_added_keyword = NULL;

	const char *video_query = "(MEDIA_TYPE=1 and MEDIA_DISPLAY_NAME LIKE \"%%%s%%\" ESCAPE (\"%s\"))";

	SEARCH_MALLOC(escape_added_keyword, strlen(keyword)*2+5, char);
	memset(escape_added_keyword, 0x0, strlen(keyword)*2+5);
	search_sql_make_keyword_bind_value(keyword, escape_added_keyword, SEARCH_SQL_BIND_TYPE_DUPLEX);

	len = strlen(escape_added_keyword) + strlen(DB_ESCAPE_CHAR) + strlen(video_query) + 1;
	search_query = (char*)calloc(1, len);

	snprintf(search_query, len, video_query, escape_added_keyword, DB_ESCAPE_CHAR);

	filter_h filter;

	ret = media_filter_create(&filter);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_create failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = media_filter_set_offset(filter, offset, limit);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_offset failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = media_filter_set_condition(filter, search_query, MEDIA_CONTENT_COLLATE_DEFAULT);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_condition failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = media_filter_set_order(filter, MEDIA_CONTENT_ORDER_ASC, MEDIA_DISPLAY_NAME, MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_order failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
			}

	ret = __search_sql_media_search_result_by_api(obj,
				offset,
				limit,
				keyword,
				filter,
				__search_media_info_cb,
				data,
				result_info);

out_func:

	SEARCH_FREE(search_query);
	SEARCH_FREE(escape_added_keyword);
	media_filter_destroy(filter);

	SEARCH_FUNC_END;

	return ret;
			}

int search_sql_music_search_result_by_api(struct search_content_object *obj,
		int offset, int limit,
		char *keyword,
		void *data,
		struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	int ret = SEARCH_RET_SEARCH_SUCCESS;
	int len = 0;
	char *search_query = NULL;
	char *escape_added_keyword = NULL;

	const char *music_query = "(MEDIA_TYPE=3 and MEDIA_TITLE LIKE \"%%%s%%\" or MEDIA_ALBUM LIKE \"%%%s%%\" or MEDIA_ARTIST LIKE \"%%%s%%\" ESCAPE (\"%s\"))";

	SEARCH_MALLOC(escape_added_keyword, strlen(keyword)*2+5, char);
	memset(escape_added_keyword, 0x0, strlen(keyword)*2+5);
	search_sql_make_keyword_bind_value(keyword, escape_added_keyword, SEARCH_SQL_BIND_TYPE_DUPLEX);

	len = strlen(escape_added_keyword) + strlen(DB_ESCAPE_CHAR) + strlen(music_query) + 1;
	search_query = (char*)calloc(1, len);

	snprintf(search_query, len, music_query, escape_added_keyword,
		escape_added_keyword, escape_added_keyword, DB_ESCAPE_CHAR);

	filter_h filter;

	ret = media_filter_create(&filter);
		if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_create failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
			goto out_func;
				}

	ret = media_filter_set_offset(filter, offset, limit);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_offset failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
				}

	ret = media_filter_set_condition(filter, search_query, MEDIA_CONTENT_COLLATE_DEFAULT);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_condition failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
			}

	ret = media_filter_set_order(filter, MEDIA_CONTENT_ORDER_ASC, MEDIA_TITLE, MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		SEARCH_DEBUG_WARNING("media_filter_set_order failed : %d\n", ret);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	ret = __search_sql_media_search_result_by_api(obj,
				offset,
				limit,
				keyword,
				filter,
				__search_sql_music_info_cb,
				data,
				result_info);

out_func:

	SEARCH_FREE(search_query);
	SEARCH_FREE(escape_added_keyword);
	media_filter_destroy(filter);

	SEARCH_FUNC_END;

	return ret;
}

ail_cb_ret_e __search_sql_menu_search_result_cb(
					const ail_appinfo_h appinfo,
					void *data)
{
	SEARCH_FUNC_START;

	struct search_cb_data *scd = (struct search_cb_data*)data;
	struct appdata *ad = scd->ad;
	struct search_content_object* obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_genlist_item_info *gen_item = NULL;
	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;

	bool is_word_in_str;
	ail_cb_ret_e ret;

	if (obj->mr_info.loop_cnt < obj->mr_info.next_point) {
		obj->mr_info.loop_cnt++;
		return AIL_CB_RET_CONTINUE;
	}

	if(obj->mr_info.next_point <
	    obj->mr_info.offset + SEARCH_CATEGORY_LIST_MORE_CNT) {

		sel_mem_info = (struct search_item_sel *)calloc(1, sizeof(struct search_item_sel));
		sel_mem_info->type = obj->contents_type;;

		/********** START : set search result to sel_mem_info buf ************/

		char *name = NULL;
		char *package = NULL;
		char *icon = NULL;

		ail_appinfo_get_str(appinfo, AIL_PROP_ICON_STR, &icon);

		if (icon)
			strncpy(sel_mem_info->icon1_path, icon,
					MAX_LENGTH_PER_PATH - 1);

		ail_appinfo_get_str(appinfo, AIL_PROP_PACKAGE_STR, &package);
		if ((package) && (strlen(package) > 0)) {

			if(strcmp(package, "org.tizen.smartsearch") == 0) {
				SEARCH_DEBUG_LOG("'smartsearch' does not appended");
				obj->mr_info.loop_cnt++;
				SEARCH_FREE(sel_mem_info);
				return AIL_CB_RET_CONTINUE;
			}

			strncpy(sel_mem_info->path, package, MAX_LENGTH_PER_PATH - 1);

			SEARCH_DEBUG_LOG("package : %s", package);
		} else {
			SEARCH_DEBUG_LOG("package : null");
		}

		ail_appinfo_get_str(appinfo, AIL_PROP_NAME_STR, &name);
		if ((name) && (strlen(name) > 0)) {
			strncpy(sel_mem_info->main_buf,
				search_markup_keyword(name, keyword, &is_word_in_str),
					MAX_LENGTH_PER_LINE - 1);

			SEARCH_DEBUG_LOG("name : %s", name);
		} else {
			SEARCH_DEBUG_LOG("name : null");
}

		if(obj->mr_info.next_point == 0) {
			/* Set group title */
			result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
														obj,
														ad->itc_grouptitle,
											    		obj,
											    NULL);
		}

		/********** END : set search result to sel_mem_info buf *************/

		gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
										obj,
										ad->itc_pslist_1line_1icon,
					sel_mem_info,
										obj->gl_result_grp);
		result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
		obj->mr_info.next_point++;
		obj->mr_info.loop_cnt++;

		ret = AIL_CB_RET_CONTINUE;

	} else if (obj->mr_info.next_point ==
	   			obj->mr_info.offset + SEARCH_CATEGORY_LIST_MORE_CNT) {

		/* Check that only "SMARTSEARCH" is left in more result */
		if(obj->mr_info.next_point == obj->mr_info.result_cnt-1) {

			char *package = NULL;

			ail_appinfo_get_str(appinfo, AIL_PROP_PACKAGE_STR, &package);
			if ((package) && (strlen(package) > 0) && (strcmp(package, "org.tizen.smartsearch") == 0)) {
				SEARCH_DEBUG_LOG("Only 'SMARTSEARCH' is left in more result");
				return AIL_CB_RET_CANCEL;
			}
		}

		/* Set more title to genlist (under 'search all') */
		sel_mem_more = (struct search_item_more_sel *)calloc(1, sizeof(struct search_item_more_sel));
		sel_mem_more->obj = obj;

		result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
												obj,
												ad->itc_listmore,
												sel_mem_more,
												obj->gl_result_grp);

		ret = AIL_CB_RET_CANCEL;
	} else {
		SEARCH_DEBUG_WARNING
		    ("__search_sql_memo_search_result_cb wrong index : %d",
		     obj->mr_info.next_point);

		ret = AIL_CB_RET_CANCEL;
	}

	SEARCH_FUNC_END;

	return ret;
}

int search_sql_menu_search_result_by_api(struct search_content_object *obj,
		int offset, int limit,
		char *keyword,
		void *data,
		struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	struct search_cb_data scd = {obj, ad, keyword, result_info};

	int cnt = 0;
	int ret = 0;

	ail_filter_h filter;
	ail_error_e rc;

	char *escape_added_keyword = NULL;
	SEARCH_MALLOC(escape_added_keyword, strlen(keyword)*2+5, char);
	memset(escape_added_keyword, 0x0, strlen(keyword)*2+5);
	search_sql_make_keyword_bind_value(keyword, escape_added_keyword, SEARCH_SQL_BIND_TYPE_DUPLEX);

	rc = ail_filter_new(&filter);
	if (rc != AIL_ERROR_OK) {
		SEARCH_DEBUG_WARNING("menu search - ail_filter_new error : %d", rc);
		return SEARCH_RET_SEARCH_FAIL;
	}

	rc = ail_filter_add_str(filter, AIL_PROP_NAME_STR, escape_added_keyword);
	if (rc != AIL_ERROR_OK) {
		SEARCH_DEBUG_WARNING("menu search - ail_filter_add_str error : %d", rc);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	rc = ail_filter_add_bool(filter, AIL_PROP_NODISPLAY_BOOL, false);
	if (rc != AIL_ERROR_OK) {
		SEARCH_DEBUG_WARNING("menu search - ail_filter_add_bool error : %d", rc);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	rc = ail_filter_count_appinfo(filter, &cnt);
	if( rc != AIL_ERROR_OK) {
		SEARCH_DEBUG_WARNING("menu search - ail_filter_count_appinfo error : %d", rc);
		ret = SEARCH_RET_SEARCH_FAIL;
		goto out_func;
	}

	SEARCH_DEBUG_LOG("ail search result : %d", cnt);
	obj->mr_info.result_cnt = cnt;

	if(cnt > 0) {
		obj->mr_info.loop_cnt = 0;
		rc = ail_filter_list_appinfo_foreach(filter, __search_sql_menu_search_result_cb, &scd);
		if( rc != AIL_ERROR_OK) {
			SEARCH_DEBUG_WARNING("menu search - ail_filter_list_appinfo_foreach error : %d", rc);
			ret = SEARCH_RET_SEARCH_FAIL;
			goto out_func;
		}

		if (obj->mr_info.next_point == 0) {
			ret = SEARCH_RET_SEARCH_NONE;
		} else {
			ret = SEARCH_RET_SEARCH_SUCCESS;
		}
	} else {
		ret = SEARCH_RET_SEARCH_NONE;
	}

out_func:
	ail_filter_destroy(filter);
	SEARCH_FREE(escape_added_keyword);
	SEARCH_FUNC_END;

	return ret;
}

bool __search_sql_browser_bookmark_result_cb(favorites_bookmark_entry_s *item, void *user_data)
{
	SEARCH_FUNC_START;
	struct search_item_sel *sel_mem_info = NULL;

	bool ret = false;
	bool is_word_in_str_main = false;
	bool is_word_in_str_sub = false;

	const char* match_str = NULL;

	struct search_cb_data_browser *scd = (struct search_cb_data_browser*)user_data;
	struct appdata *ad = scd->ad;
	struct search_content_object *obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_genlist_item_info *gen_item = NULL;
	struct search_more_result *info = scd->info;

	if (info->loop_cnt < info->next_point) {
		info->loop_cnt++;
		return true;
	}

	if (info->next_point <
	    info->offset + SEARCH_CATEGORY_LIST_MORE_CNT) {

		sel_mem_info = (struct search_item_sel *)calloc(1, sizeof(struct search_item_sel));
		sel_mem_info->type = obj->contents_type;;

		/********** START : set search result to sel_mem_info buf ************/

		if (item != NULL && item->title) {
			match_str = search_markup_keyword(item->title, keyword, &is_word_in_str_main);
			strncpy(sel_mem_info->main_buf,
					match_str,
					MAX_LENGTH_PER_LINE - 1);
		}

		if (item != NULL && item->address) {
			match_str = search_markup_keyword(item->address, keyword, &is_word_in_str_sub);
			strncpy(sel_mem_info->sub_buf,
					match_str,
					MAX_LENGTH_PER_LINE - 1);
			strncpy(sel_mem_info->path,
					item->address,
					MAX_LENGTH_PER_PATH - 1);
		}

		if (is_word_in_str_main == true || is_word_in_str_sub == true) {
			if (obj->gl_result_grp == NULL ) {
				/* Set group title */
				result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
															obj,
															ad->itc_grouptitle,
															obj,
															NULL);
			}

			if (obj->obj_icon_path)
				strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
						MAX_LENGTH_PER_PATH - 1);

			strncpy(sel_mem_info->icon2_path,
					SEARCH_FAVORITE_ICON,
					MAX_LENGTH_PER_PATH - 1);

			gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
											obj,
											ad->itc_pslist_2line_2icon,
								sel_mem_info,
											obj->gl_result_grp);
			result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
		}

		/********** END : set search result to sel_mem_info buf *************/
		info->next_point++;
		info->loop_cnt++;

		ret = true;
	} else if (info->next_point ==
	   			info->offset + SEARCH_CATEGORY_LIST_MORE_CNT) {
		info->offset = info->next_point;
		ret = false;
	} else {
		SEARCH_DEBUG_WARNING
		    ("__search_sql_browser_bookmark_result_cb wrong index : %d",
		     info->next_point);

		ret = false;
	}
	SEARCH_FUNC_END;
	return ret;

}

bool __search_sql_browser_history_result_cb(favorites_history_entry_s *item, void *user_data)
{
	SEARCH_FUNC_START;
	struct search_item_sel *sel_mem_info = NULL;

	bool ret = false;
	bool is_word_in_str_main = false;

	const char* match_str = NULL;

	struct search_cb_data_browser *scd = (struct search_cb_data_browser*)user_data;
	struct appdata *ad = scd->ad;
	struct search_content_object *obj = scd->obj;
	struct search_genlist_info *result_info = scd->result_info;
	char *keyword = scd->keyword;

	struct search_more_result *info = scd->info;
	struct search_genlist_item_info *gen_item = NULL;

	if (info->loop_cnt < info->next_point) {
		info->loop_cnt++;
		return true;
	}

	if (info->next_point <
	    info->offset + SEARCH_CATEGORY_LIST_MORE_CNT) {

		sel_mem_info = (struct search_item_sel *)calloc(1, sizeof(struct search_item_sel));
		sel_mem_info->type = obj->contents_type;;

		/********** START : set search result to sel_mem_info buf ************/

		if (item != NULL && item->address) {
					SEARCH_DEBUG_WARNING("BROWSER : %s %s", item->address, keyword);
			match_str = search_markup_keyword(item->address, keyword, &is_word_in_str_main);
		}

		if (is_word_in_str_main != false ) {
			if (obj->gl_result_grp == NULL ) {
				/* Set group title */
			result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
														obj,
														ad->itc_grouptitle,
														obj,
														NULL);
			}

			if (obj->obj_icon_path)
				strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
						MAX_LENGTH_PER_PATH - 1);

			if (item->address)
				strncpy(sel_mem_info->path, item->address, MAX_LENGTH_PER_PATH - 1);

			strncpy(sel_mem_info->main_buf,
					match_str,
					MAX_LENGTH_PER_LINE - 1);

			gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
											obj,
											ad->itc_pslist_1line_1icon,
					sel_mem_info,
											obj->gl_result_grp);
			result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
		}

		info->next_point++;
		info->loop_cnt++;

		ret = true;

	} else if (info->next_point ==
	   			info->offset + SEARCH_CATEGORY_LIST_MORE_CNT) {
		info->offset = info->next_point;
		ret = false;
	} else {
		SEARCH_DEBUG_WARNING
		    ("__search_sql_browser_history_result_cb wrong index : %d",
		     info->next_point);

		ret = false;
	}

	SEARCH_FUNC_END;
	return ret;
}

int search_sql_browser_search_result_by_api(struct search_content_object *obj,
		int offset, int limit,
		char *keyword,
		void *data,
		struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	struct appdata *ad = (struct appdata *)data;
	struct search_cb_data_browser scd_browser = {obj, ad, keyword, result_info, NULL};
	struct search_cb_data_browser scd_history = {obj, ad, keyword, result_info, NULL};

	static struct search_more_result browser_result[2];
	struct search_item_more_sel *sel_mem_more = NULL;

	int ret = 0;
	int bookmark_cnt = 0, history_cnt = 0;
	int bookmark_ret = 0, history_ret = 0;

	if (offset == 0) {
		/* offset 0 means first search result, so we initialize struct */
		memset(browser_result, 0x0, sizeof(browser_result));

		favorites_bookmark_get_count(&browser_result[0].result_cnt);
		favorites_history_get_count(&browser_result[1].result_cnt);

		bookmark_cnt = browser_result[0].result_cnt;
		history_cnt = browser_result[1].result_cnt;
		browser_result[0].ad = browser_result[1].ad = ad;
		obj->gl_result_grp = NULL;

		if ( bookmark_cnt <= 0 && history_cnt <= 0 )
			return SEARCH_RET_SEARCH_NONE;
	} else {
		bookmark_cnt = browser_result[0].result_cnt;
		history_cnt = browser_result[1].result_cnt;
		browser_result[0].loop_cnt = 0;
		browser_result[1].loop_cnt = 0;
	}

	if (offset >= bookmark_cnt + history_cnt) {
		/* We need not to search */
		return SEARCH_RET_SEARCH_NONE;
	}

	scd_browser.info = &browser_result[0];
	scd_history.info = &browser_result[1];

	/* offset check */
	if (bookmark_cnt > browser_result[0].next_point )
		bookmark_ret = favorites_bookmark_foreach(__search_sql_browser_bookmark_result_cb,
												&scd_browser);
	if (history_cnt > browser_result[1].next_point )
		history_ret = favorites_history_foreach(__search_sql_browser_history_result_cb,
												&scd_history);

	if (bookmark_ret == FAVORITES_ERROR_DB_FAILED
		&& history_ret == FAVORITES_ERROR_DB_FAILED ) {
		ret = SEARCH_RET_SEARCH_FAIL;
	}
	else if (obj->gl_result_grp == NULL) {
		/* It means we can't get matched results */
		ret = SEARCH_RET_SEARCH_NONE;
	}
	else {
		/* Make More button */
		if (browser_result[0].next_point < bookmark_cnt
			|| browser_result[1].next_point < history_cnt) {
		sel_mem_more = (struct search_item_more_sel *)calloc(1, sizeof(struct search_item_more_sel));
		sel_mem_more->obj = obj;

			result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
													obj,
													ad->itc_listmore,
													sel_mem_more,
													obj->gl_result_grp);
		}

		/* return check */
		obj->mr_info.next_point = browser_result[0].next_point
								+ browser_result[1].next_point;
		ret = SEARCH_RET_SEARCH_SUCCESS;
	}

	return ret;

	SEARCH_FUNC_END;
}

int search_sql_msg_search_result_by_api(struct search_content_object *obj,
		int offset, int limit,
		char *keyword,
		void *data,
		struct search_genlist_info *result_info)
{
	SEARCH_FUNC_START;

	int loop_cnt = 0;
	int ret = 0;

	struct search_item_sel *sel_mem_info = NULL;
	struct search_item_more_sel *sel_mem_more = NULL;
	struct appdata *ad = (struct appdata *)data;
	struct search_genlist_item_info *gen_item = NULL;

	bool is_word_in_str;

	int i = 0;
	int j = 0;
	bool is_more_btn = false;
	bool is_word_in_title = false;

	msg_error_t err = MSG_SUCCESS;
	msg_struct_list_s msgList;
	msg_struct_t search_s = NULL;
	msg_struct_list_s *addr_list = NULL;

	search_s = msg_create_struct(MSG_STRUCT_SEARCH_CONDITION);

	err = msg_set_int_value(search_s, MSG_SEARCH_CONDITION_FOLDERID_INT, MSG_ALLBOX_ID);
	if (err != MSG_SUCCESS) {
		SEARCH_DEBUG_WARNING("Fail to make search struct : MSG_SEARCH_CONDITION_FOLDERID_INT");
		return SEARCH_RET_SEARCH_FAIL;
	};

	err = msg_set_int_value(search_s, MSG_SEARCH_CONDITION_MSGTYPE_INT, MSG_TYPE_INVALID);
	if (err != MSG_SUCCESS) {
		SEARCH_DEBUG_WARNING("Fail to make search struct : MSG_SEARCH_CONDITION_MSGTYPE_INT");
		return SEARCH_RET_SEARCH_FAIL;
	}

	err = msg_set_str_value(search_s, MSG_SEARCH_CONDITION_SEARCH_VALUE_STR, keyword, strlen(keyword));
	if (err != MSG_SUCCESS) {
		SEARCH_DEBUG_WARNING("Fail to make search struct : MSG_SEARCH_CONDITION_SEARCH_VALUE_STR");
		return SEARCH_RET_SEARCH_FAIL;
	}

	err = msg_search_message(ad->msg_handle, search_s, offset, limit, &msgList);

	if (err != MSG_SUCCESS && err != MSG_ERR_DB_NORECORD) {
		SEARCH_DEBUG_WARNING("msg_search_message is failed : %d", err);
		return SEARCH_RET_SEARCH_FAIL;
	}

	if (msgList.nCount <= 0) {
		SEARCH_DEBUG_WARNING("no msg search result");
		ret = SEARCH_RET_SEARCH_NONE;
	}  else  {

		/* Set group title */
		if (offset == 0) {
			/* Set group title */
			result_info->group = search_add_genlist_item(SEARCH_GENLIST_GROUP,
										obj,
										ad->itc_grouptitle,
						    			obj,
										NULL);
		}

		/* Set search result to genlist */

		if(msgList.nCount < limit) {
			loop_cnt = msgList.nCount;
			is_more_btn = false;
		} else {
			loop_cnt = limit-1;
			is_more_btn = true;
		}

		/********** START : set search result to sel_mem_info buf ************/

		for (i = 0; i < loop_cnt; i++) {
			int ret = 0;
			const char *text_1line = NULL;
			const char *markup_keyword = NULL;
			const char *img_path = NULL;

			int msg_type = 0;
			int	contact_id = -1;
			int message_id = -1;

			char subject[MAX_SUBJECT_LEN] = {0};
			char str_number[MAX_ADDRESS_VAL_LEN] = {0,};
			char str_name[MAX_DISPLAY_NAME_LEN] = {0,};
			char msg_text[MAX_MSG_TEXT_LEN] = {0,};

			CTSstruct *contact = NULL;

			sel_mem_info = (struct search_item_sel *)
							calloc(1,sizeof(struct search_item_sel));
			sel_mem_info->type = obj->contents_type;;

			err = msg_get_int_value(msgList.msg_struct_info[i], MSG_MESSAGE_ID_INT, &message_id);
			if (err != MSG_SUCCESS) {
				SEARCH_DEBUG_WARNING("Fail to make search struct : MSG_MESSAGE_ID_INT %d", err);
				goto out_func;
			}
			snprintf(sel_mem_info->main_id, MAX_LENGTH_PER_ID - 1, "%d", message_id);

			err = msg_get_int_value(msgList.msg_struct_info[i], MSG_MESSAGE_TYPE_INT, &msg_type);
			if (err != MSG_SUCCESS) {
				SEARCH_DEBUG_WARNING("Fail to make search struct : MSG_MESSAGE_TYPE_INT %d", err);
				goto out_func;
			}

			SEARCH_DEBUG_LOG("mms : %d", msg_type);

			/* Set 1st line string */
			if (msg_type == MSG_TYPE_MMS) {
				/* MMS case */
				err = msg_get_str_value(msgList.msg_struct_info[i], MSG_MESSAGE_SUBJECT_STR,
									subject, MAX_SUBJECT_LEN);

				if (err != MSG_SUCCESS) {
					SEARCH_DEBUG_WARNING("Fail to get sms context %d", err);
					goto out_func;
				}

				text_1line = subject;
				markup_keyword = search_markup_keyword(text_1line,
											keyword,
					     								&is_word_in_title);

				if (!is_word_in_title) {
					memset(msg_text, 0x00, sizeof(msg_text));
					err = msg_get_str_value(msgList.msg_struct_info[i],
										MSG_MESSAGE_MMS_TEXT_STR,
										msg_text,
										sizeof(msg_text));

					if (err != MSG_SUCCESS) {
						SEARCH_DEBUG_WARNING("Fail to get mms context %d", err);
						goto out_func;
					}

					text_1line = msg_text;

					markup_keyword = search_markup_keyword(text_1line,
							keyword,
	     													&is_word_in_title);
			}

				SEARCH_DEBUG_LOG("1st string : %s", text_1line);

				if ((text_1line) && (strlen(text_1line) > 0 ))
					strncpy(sel_mem_info->main_buf, markup_keyword, MAX_LENGTH_PER_LINE - 1);
			} else {
				/* SMS Case */
				err = msg_get_str_value(msgList.msg_struct_info[i],
									MSG_MESSAGE_SMS_DATA_STR,
									msg_text,
									sizeof(msg_text));

				if (err != MSG_SUCCESS) {
					SEARCH_DEBUG_WARNING("Fail to get sms context %d", err);
					goto out_func;
				}

				text_1line = msg_text;

				markup_keyword = search_markup_keyword(text_1line,
											keyword,
					     								&is_word_in_title);
				SEARCH_DEBUG_LOG("1st string : %s", text_1line);

				if ((text_1line) && (strlen(text_1line) > 0))
					strncpy(sel_mem_info->main_buf, markup_keyword, MAX_LENGTH_PER_LINE - 1);
			}

			err = msg_get_list_handle(msgList.msg_struct_info[i],
								MSG_MESSAGE_ADDR_LIST_STRUCT,
								(void **)&addr_list);

			if (err != MSG_SUCCESS) {
				SEARCH_DEBUG_WARNING("Fail to get address list handle! %d", err);
				goto out_func;
			}

			/* Set 2nd line string */
			for ( j=0; j<addr_list->nCount; j++) {
				memset(str_number, 0, sizeof(str_number));
				memset(str_name, 0, sizeof(str_name));
				contact_id = -1;
				is_word_in_str = false;

				/* First We set text_2line to first address data */
				err = msg_get_str_value(addr_list->msg_struct_info[j],
									MSG_ADDRESS_INFO_ADDRESS_VALUE_STR,
									str_number,
									MAX_ADDRESS_VAL_LEN);

				if (err != MSG_SUCCESS) {
					SEARCH_DEBUG_WARNING("Fail to get data! : MSG_ADDRESS_INFO_DISPLAYNAME_STR %d", err);
					goto out_func;
				}

				err = msg_get_str_value(addr_list->msg_struct_info[j],
									MSG_ADDRESS_INFO_DISPLAYNAME_STR,
									str_name,
									MAX_DISPLAY_NAME_LEN);

				if (err != MSG_SUCCESS) {
					SEARCH_DEBUG_WARNING("Fail to get data! : MSG_ADDRESS_INFO_ADDRESS_VALUE_STR %d", err);
					goto out_func;
				}

				err = msg_get_int_value(addr_list->msg_struct_info[j],
									MSG_ADDRESS_INFO_CONTACT_ID_INT,
									&contact_id);

				if (err != MSG_SUCCESS) {
					SEARCH_DEBUG_WARNING("Fail to get data! : MSG_ADDRESS_INFO_CONTACT_ID_INT %d", err);
					goto out_func;
				}

				/* Set text_2line */
				markup_keyword = search_markup_keyword(str_number, keyword, &is_word_in_str);

				if( 0 == j || true == is_word_in_str) {
					if (strlen(str_name) > 0) {
						strncpy(sel_mem_info->sub_buf, str_name, MAX_LENGTH_PER_LINE - 1);
				} else {
						strncpy(sel_mem_info->sub_buf, markup_keyword, MAX_LENGTH_PER_LINE - 1);
				}

					/* Set Thumbnail Image */
					if (contact_id == -1) {
						strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
								MAX_LENGTH_PER_PATH - 1);
					}
					else {
						ret = contacts_svc_get_contact(contact_id, &contact);

						if (ret == CTS_SUCCESS) {
							CTSvalue* value = NULL;
							ret = contacts_svc_struct_get_value(contact, CTS_CF_BASE_INFO_VALUE, &value);

							if (ret == CTS_SUCCESS)
								img_path = contacts_svc_value_get_str(value, CTS_BASE_VAL_IMG_PATH_STR);

							if (img_path != NULL)
								strncpy(sel_mem_info->icon1_path , img_path,
										MAX_LENGTH_PER_PATH -1 );
							else if (obj->obj_icon_path)
								strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
										MAX_LENGTH_PER_PATH - 1);
						}
						else if (obj->obj_icon_path)
								strncpy(sel_mem_info->icon1_path, obj->obj_icon_path,
										MAX_LENGTH_PER_PATH - 1);
					}
				}

				if ((is_word_in_title) || (is_word_in_str)) {
					break;
				}
			}

			SEARCH_DEBUG_LOG("sel_mem_info->sub_buf : %s", sel_mem_info->sub_buf);

			gen_item = search_add_genlist_item(SEARCH_GENLIST_ITEM,
							obj,
							ad->itc_pslist_2line_1icon,
							sel_mem_info,
							obj->gl_result_grp);

			result_info->itemlist = g_list_append(result_info->itemlist, (void*)gen_item);
		}

		msg_release_list_struct(&msgList);

		/* Set more title to genlist (under 'search all') */
		if (is_more_btn) {
			obj->mr_info.next_point = offset + limit - 1;

			sel_mem_more = (struct search_item_more_sel *)
			    		calloc(1,sizeof(struct search_item_more_sel));
			sel_mem_more->obj = obj;

			result_info->more = search_add_genlist_item(SEARCH_GENLIST_MORE,
										obj,
										ad->itc_listmore,
										sel_mem_more,
										obj->gl_result_grp);
		}

		ret = SEARCH_RET_SEARCH_SUCCESS;
	}

out_func:
	SEARCH_FUNC_END;
	return ret;
}

void search_sql_update_result_object(struct search_content_object* result_obj, sqlite3* db_handle, sqlite3_stmt** db_sql)
{
	SEARCH_FUNC_START;

	int rc = 0;

	sqlite3_stmt *sql_stmt = db_sql[SEARCH_STMT_UPDATE_OBJECT_INFO];
	SEARCH_RET_IF(sql_stmt==NULL);

	search_sql_bind_text(db_handle, sql_stmt, 1, result_obj->name);
	search_sql_bind_int(db_handle, sql_stmt, 2, result_obj->string_type);
	search_sql_bind_int(db_handle, sql_stmt, 3, result_obj->visible);
	search_sql_bind_int(db_handle, sql_stmt, 4, result_obj->order);
	search_sql_bind_int(db_handle, sql_stmt, 5, result_obj->contents_type);
	search_sql_bind_int(db_handle, sql_stmt, 6, result_obj->category_type);
	search_sql_bind_text(db_handle, sql_stmt, 7, result_obj->def_pkg_name);
	search_sql_bind_int(db_handle, sql_stmt, 8, result_obj->contents_type);

	rc = sqlite3_step(sql_stmt);

	if (rc == SQLITE_OK)
		SEARCH_DEBUG_LOG("INSERT STEP SUCCESS");

	search_sql_stmt_init(sql_stmt);

	SEARCH_FUNC_END;

}
